﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=4.2
@EndOfDesignText@
'version 1.01
Sub Process_Globals
	Private fx As JFX
	Public DB_REAL, DB_INTEGER, DB_BLOB, DB_TEXT As String
	DB_REAL = "REAL"
	DB_INTEGER = "INTEGER"
	DB_BLOB = "BLOB"
	DB_TEXT = "TEXT"
	Dim HtmlCSS As String = "table {width: 100%;border: 1px solid #cef;text-align: left; }" _
		& " th { font-weight: bold;	background-color: #acf;	border-bottom: 1px solid #cef; }" _ 
		& "td,th {	padding: 4px 5px; }" _
		& ".odd {background-color: #def; } .odd td {border-bottom: 1px solid #cef; }" _
		& "a { text-decoration:none; color: #000;}"
End Sub

Private Sub EscapeField(f As String) As String
	Return "[" & f & "]"
End Sub

'Crea una nuova tabella con il nome specificato.
'FieldsAndTypes - Una mappa con i nomi "fields" dei campi come chiavi "keys" e i tipi come valori "types as values"
'Puoi usare le costanti DB_... per i tipi.
'PrimaryKey - La colonna che sarà la chiave primaria. Passa una stringa vuota se non necessario.
Public Sub CreateTable(SQL As SQL, TableName As String, FieldsAndTypes As Map, PrimaryKey As String)
	Dim sb As StringBuilder
	sb.Initialize
	sb.Append("(")
	For i = 0 To FieldsAndTypes.Size - 1
		Dim field, ftype As String
		field = FieldsAndTypes.GetKeyAt(i)
		ftype = FieldsAndTypes.GetValueAt(i)
		If i > 0 Then sb.Append(", ")
		sb.Append(EscapeField(field)).Append(" ").Append(ftype)
		If field = PrimaryKey Then sb.Append(" PRIMARY KEY")
	Next
	sb.Append(")")
	Dim query As String
	query = "CREATE TABLE IF NOT EXISTS " & EscapeField(TableName) & " " & sb.ToString
	Log("CreateTable: " & query)
	SQL.ExecNonQuery(query)
End Sub

'Elimina la tabella selezionata
Public Sub DropTable(SQL As SQL, TableName As String)
	Dim query As String
	query = "DROP TABLE IF EXISTS " & EscapeField(TableName)
	Log("DropTable3: " & query)
	SQL.ExecNonQuery(query)
End Sub

'Inserisce i dati nella tabella.
'ListOfMaps - Un elenco con le mappe come elementi. Ogni mappa rappresenta un record in cui le chiavi della mappa sono i nomi delle colonne
'e i valori delle mappe sono i valori.
'Nota che dovresti creare una nuova mappa per ogni record (questo può essere fatto chiamando Dim per rimodellare la mappa).
Public Sub InsertMaps(SQL As SQL, TableName As String, ListOfMaps As List)
	Dim sb, columns, values As StringBuilder
	'Small check for a common error where the same map is used in a loop
	If ListOfMaps.Size > 1 And ListOfMaps.Get(0) = ListOfMaps.Get(1) Then
		Log("Same Map found twice in list. Each item in the list should include a different map object.")
		Return
	End If
	SQL.BeginTransaction
	Try
		For i1 = 0 To ListOfMaps.Size - 1
			sb.Initialize
			columns.Initialize
			values.Initialize
			Dim listOfValues As List
			listOfValues.Initialize
			sb.Append("INSERT INTO [" & TableName & "] (")
			Dim m As Map
			m = ListOfMaps.Get(i1)
			For i2 = 0 To m.Size - 1
				Dim col As String
				Dim value As Object	
				col = m.GetKeyAt(i2)
				value = m.GetValueAt(i2)
				If i2 > 0 Then
					columns.Append(", ")
					values.Append(", ")
				End If
				columns.Append(EscapeField(col))
				
				values.Append("?")
				listOfValues.Add(value)
			Next
			sb.Append(columns.ToString)
			sb.Append(") VALUES (")
			sb.Append(values.ToString)
			sb.Append(")")
			If i1 = 0 Then Log("InsertMaps (first query out of " & ListOfMaps.Size & "): " & sb.ToString)
			SQL.ExecNonQuery2(sb.ToString, listOfValues)
		Next
		SQL.TransactionSuccessful
	Catch
		Log(LastException)
		SQL.Rollback
	End Try
End Sub

'aggiorna un singolo campo in un record
'Field è il nome della colonna
Public Sub UpdateRecord(SQL As SQL, TableName As String, Field As String, NewValue As Object, _
	WhereFieldEquals As Map)
	Dim sb As StringBuilder
	sb.Initialize
	sb.Append("UPDATE ").Append(EscapeField(TableName)).Append(" SET ").Append(EscapeField(Field)) _
		.Append(" = ? WHERE ")
	If WhereFieldEquals.Size = 0 Then
		Log("WhereFieldEquals map empty!")
		Return
	End If
	Dim args As List
	args.Initialize
	args.Add(NewValue)
	For i = 0 To WhereFieldEquals.Size - 1
		If i > 0 Then sb.Append(" AND ")
		sb.Append(EscapeField(WhereFieldEquals.GetKeyAt(i))).Append(" = ?")
		args.Add(WhereFieldEquals.GetValueAt(i))
	Next
	Log("UpdateRecord: " & sb.ToString)
	SQL.ExecNonQuery2(sb.ToString, args)
End Sub

' aggiorna più campi in un record
' nella mappa Campi "Fields map" le chiavi sono i nomi delle colonne
Public Sub UpdateRecord2(SQL As SQL, TableName As String, Fields As Map, WhereFieldEquals As Map)
	If WhereFieldEquals.Size = 0 Then
		Log("WhereFieldEquals map empty!")
		Return
	End If
	If Fields.Size = 0 Then
		Log("Fields empty")
		Return
	End If
	Dim sb As StringBuilder
	sb.Initialize
	sb.Append("UPDATE ").Append(EscapeField(TableName)).Append(" SET ")
	Dim args As List
	args.Initialize
	For i=0 To Fields.Size-1
		If i<>Fields.Size-1 Then
			sb.Append(EscapeField(Fields.GetKeyAt(i))).Append("=?,")
		Else
			sb.Append(EscapeField(Fields.GetKeyAt(i))).Append("=?")
		End If
		args.Add(Fields.GetValueAt(i))
	Next
    
	sb.Append(" WHERE ")
	For i = 0 To WhereFieldEquals.Size - 1
		If i > 0 Then 
			sb.Append(" AND ")
		End If
		sb.Append(EscapeField(WhereFieldEquals.GetKeyAt(i))).Append(" = ?")
		args.Add(WhereFieldEquals.GetValueAt(i))
	Next
	Log("UpdateRecord: " & sb.ToString)
	SQL.ExecNonQuery2(sb.ToString, args)
End Sub


'Esegue la query e restituisce il risultato come un elenco di array.
'Ogni elemento nell'elenco è un array di stringhe.
'StringArgs - Valori per sostituire i punti interrogativi nella query. Passa Null se non necessario.
'Limite - Limita i risultati. Passa 0 per tutti i risultati.
Public Sub ExecuteMemoryTable(SQL As SQL, Query As String, StringArgs() As String, Limit As Int) As List
	Dim cur As ResultSet
	If StringArgs = Null Then 
		Dim StringArgs(0) As String
	End If
	cur = SQL.ExecQuery2(Query, StringArgs)
	Log("ExecuteMemoryTable: " & Query)
	Dim table As List
	table.Initialize
	Do While cur.NextRow
		Dim values(cur.ColumnCount) As String
		For col = 0 To cur.ColumnCount - 1
			values(col) = cur.GetString2(col)
		Next
		table.Add(values)
		If Limit > 0 And table.Size >= Limit Then Exit
	Loop
	cur.Close
	Return table
End Sub

'Esegue la query e restituisce una mappa con i nomi delle colonne come chiavi
'e i primi valori del record sono i valori delle voci.
'Le chiavi sono in minuscolo.
'Restituisce una mappa non inizializzata se non ci sono risultati.
Public Sub ExecuteMap(SQL As SQL, Query As String, StringArgs() As String) As Map
	Dim res As Map
	Dim cur As ResultSet
	If StringArgs <> Null Then 
		cur = SQL.ExecQuery2(Query, StringArgs)
	Else
		cur = SQL.ExecQuery(Query)
	End If
	Log("ExecuteMap: " & Query)
	If cur.NextRow = False Then
		Log("No records found.")
		Return res
	End If
	res.Initialize
	For i = 0 To cur.ColumnCount - 1
		res.Put(cur.GetColumnName(i).ToLowerCase, cur.GetString2(i))
	Next
	cur.Close
	Return res
End Sub

'Esegue la query e riempie l'elenco con i valori nella prima colonna.
Public Sub ExecuteList(SQL As SQL, Query As String, StringArgs() As String, Limit As Int, List1 As List)
	List1.Clear
	Dim Table As List
	Table = ExecuteMemoryTable(SQL, Query, StringArgs, Limit)
	If Table.Size = 0 Then Return
	Dim Cols() As String
	For i = 0 To Table.Size - 1
		Cols = Table.Get(i)
		List1.Add(Cols(0))
	Next
End Sub

Public Sub ExecuteTableView(SQL As SQL, Query As String, StringArgs() As String, Limit As Int, _
	TableView1 As TableView)
	TableView1.Items.Clear
	Dim cur As ResultSet
	If StringArgs = Null Then 
		Dim StringArgs(0) As String
	End If
	cur = SQL.ExecQuery2(Query, StringArgs)
	Dim cols As List
	cols.Initialize
	For i = 0 To cur.ColumnCount - 1
		cols.Add(cur.GetColumnName(i))
	Next
	TableView1.SetColumns(cols)
	Do While cur.NextRow
		Dim values(cur.ColumnCount) As String
		For col = 0 To cur.ColumnCount - 1
			values(col) = cur.GetString2(col)
		Next
		TableView1.Items.Add(values)
		If Limit > 0 And TableView1.Items.Size >= Limit Then Exit
	Loop
	cur.Close
End Sub

'Crea un testo html che visualizza i dati in una tabella.
'Lo stile della tabella può essere modificato modificando la variabile HtmlCSS.
Public Sub ExecuteHtml(SQL As SQL, Query As String, StringArgs() As String, Limit As Int) As String
	Dim cur As ResultSet
	If StringArgs <> Null Then 
		cur = SQL.ExecQuery2(Query, StringArgs)
	Else
		cur = SQL.ExecQuery(Query)
	End If
	Log("ExecuteHtml: " & Query)
	Dim sb As StringBuilder
	sb.Initialize
	sb.Append("<html><body>").Append(CRLF)
	sb.Append("<style type='text/css'>").Append(HtmlCSS).Append("</style>").Append(CRLF)
	sb.Append("<table><thead><tr>").Append(CRLF)
	For i = 0 To cur.ColumnCount - 1
		sb.Append("<th>").Append(cur.GetColumnName(i)).Append("</th>")
	Next
	sb.Append("</thead>")
	
'	For i = 0 To cur.ColumnCount - 1
'		If i = 1 Then
'			sb.Append("<th style='width:200px;'>").Append(cur.GetColumnName(i)).Append("</th>")
'		Else
'			sb.Append("<th>").Append(cur.GetColumnName(i)).Append("</th>")
'		End If
'	Next
		
	sb.Append("</tr>").Append(CRLF)
	Dim row As Int
	Do While cur.NextRow
		If row Mod 2 = 0 Then
			sb.Append("<tr>")
		Else
			sb.Append("<tr class='odd'>")
		End If
		For i = 0 To cur.ColumnCount - 1
			sb.Append("<td>")
			sb.Append(cur.GetString2(i))
			sb.Append("</td>")
		Next
		sb.Append("</tr>").Append(CRLF)
		row = row + 1
	Loop
	cur.Close
	sb.Append("</table></body></html>")
	Return sb.ToString
End Sub

Public Sub DeleteRecord(SQL As SQL, TableName As String, WhereFieldEquals As Map)
   Dim sb As StringBuilder
   sb.Initialize
   sb.Append("DELETE FROM [").Append(TableName).Append("] WHERE ")
   If WhereFieldEquals.Size = 0 Then
      Log("WhereFieldEquals map empty!")
      Return
   End If
   Dim args As List
   args.Initialize
   For i = 0 To WhereFieldEquals.Size - 1
      If i > 0 Then sb.Append(" AND ")
      sb.Append("[").Append(WhereFieldEquals.GetKeyAt(i)).Append("] = ?")
      args.Add(WhereFieldEquals.GetValueAt(i))
   Next
   Log("DeleteRecord: " & sb.ToString)
   SQL.ExecNonQuery2(sb.ToString, args)
End Sub

Public Sub LoadData(SQL As SQL, Query As String, StringArgs() As String, Limit As Int, _
	TableView1 As TableView)
	
	TableView1.Items.Clear
	
	Dim OrgYr, LastYr, ThisYr, ClrValue As Int
	

	DateTime.DateFormat = "dd-MM-yyyy"
		
	ThisYr = DateTime.GetYear(DateTime.Now)
	
	Dim cur As ResultSet
	If StringArgs = Null Then 
		Dim StringArgs(0) As String
	End If
	cur = SQL.ExecQuery2(Query, StringArgs)
	
	
	Do While cur.NextRow
		Dim values(cur.ColumnCount) As Object
		For col = 0 To cur.ColumnCount - 1
			' examine dates to work out color for each row
			' col 6 = LastUsedDate; col 7 = OriginalDate
'			LastYr = DateTime.GetYear(DateTime.DateParse(cur.GetString2(6)))			
'			OrgYr = DateTime.GetYear(DateTime.DateParse(cur.GetString2(7)))
			
			If OrgYr < ThisYr And LastYr < ThisYr Then ClrValue = 1		' Normal Black
			If OrgYr < ThisYr And LastYr = ThisYr Then ClrValue = 2		' BLUE Bold
			If OrgYr = ThisYr And LastYr = ThisYr Then
				If DateTime.DateParse(cur.GetString2(6)) = DateTime.DateParse(cur.GetString2(7)) Then
					ClrValue = 3 	' Black Bold
				Else
					ClrValue = 4	' RED Bold
				End If
			End If
			' 1.[OLD] BLUE BOLD for OriginalDate ( Year - Not this current Year) <> LastUsedDate (Year) and LastUsedDate (Year) = Current Year
			
			' 2.[NEW Current Year] BLACK BOLD for OriginalDate (Year) = Current Year and LastUsedDate (Year) = Cur Year
			
			Select col
				Case 0, 1
					Dim lbl As Label
					lbl.Initialize("")
					lbl.PrefWidth = TableView1.GetColumnWidth(col)
					Select ClrValue
						Case 1
							lbl.Style = "-fx-text-fill: black;"
						Case 2
							lbl.Style = "-fx-text-fill: blue; -fx-font-weight: bold;"							
						Case 3
							lbl.Style = "-fx-text-fill: black; -fx-font-weight: bold;"							
						Case 4
							lbl.Style = "-fx-text-fill: red; -fx-font-weight: bold;"
					End Select
'					If col = 6 Then lbl.Style = "-fx-text-fill: purple;"
					lbl.Text = cur.GetString2(col)
					
					values(col) = lbl
				Case 2, 4, 5
					Dim lbl As Label
					lbl.Initialize("")
					lbl.PrefWidth = TableView1.GetColumnWidth(col)
'					lbl.Alignment = "CENTER_RIGHT"
					lbl.Alignment = "CENTER"
'					lbl.Style = "-fx-padding: 0 8 0 1"
					Select ClrValue
						Case 1
							lbl.Style = "-fx-text-fill: black; -fx-padding: 0 8 0 1"
						Case 2
							lbl.Style = "-fx-text-fill: blue; -fx-font-weight: bold; -fx-padding: 0 8 0 1"							
						Case 3
							lbl.Style = "-fx-text-fill: black; -fx-font-weight: bold; -fx-padding: 0 8 0 1"							
						Case 4
							lbl.Style = "-fx-text-fill: red; -fx-font-weight: bold;-fx-padding: 0 8 0 1"
					End Select
					lbl.Text = cur.GetString2(col)
					values(col) = lbl									
				Case Else
					Dim lbl As Label
					lbl.Initialize("")
					lbl.PrefWidth = TableView1.GetColumnWidth(col)
					lbl.Alignment = "CENTER"
					Select ClrValue
						Case 1
							lbl.Style = "-fx-text-fill: black;"
						Case 2
							lbl.Style = "-fx-text-fill: blue; -fx-font-weight: bold;"							
						Case 3
							lbl.Style = "-fx-text-fill: black; -fx-font-weight: bold;"							
						Case 4
							lbl.Style = "-fx-text-fill: red; -fx-font-weight: bold;"
					End Select
'					If col = 1 Then lbl.Style = "-fx-text-fill: red; -fx-font-weight: bold;"
					lbl.Text = cur.GetString2(col)
					values(col) = lbl
			End Select
		Next
		TableView1.Items.Add(values)
		If Limit > 0 And TableView1.Items.Size >= Limit Then Exit
	Loop
	cur.Close
End Sub